//
//  BLElib.m
//  BLElib
//
//  Created by fankyo on 13-3-27.
//  Copyright (c) 2013年 fankyo. All rights reserved.
//

#import "BLElib.h"
#define LOG 1


#define QIQU_SERVICE_SHORT_UUID {0x18,0xf0}
#define QIQU_CHARACTERISTIC_TX_UUID {0x2a,0xf1}
#define QIQU_CHARACTERISTIC_RX_UUID {0x2a,0xf0}


@implementation BLElib
@synthesize centralManager;
@synthesize connectedPeripheral,connectedService,msgCharacteristic;
@synthesize BLEDelegate;
@synthesize txDelayCounter;

#pragma mark -
#pragma mark Init
/****************************************************************************/
/*								Init										*/
/****************************************************************************/
- (id)init
{
    self = [super init];
    if (self) {
        if(LOG)NSLog(@"Init a central!");
        self.connectedPeripheral = nil;
        self.connectedService = nil;
        self.msgCharacteristic = nil;
        self.txDelayCounter = 0;
        centralManager = [[CBCentralManager alloc] initWithDelegate:self queue:dispatch_get_main_queue()];

	}
    return self;
}




#pragma mark -
#pragma mark Discovery
/****************************************************************************/
/*								Discovery                                   */
/****************************************************************************/
- (void) startScanningForUUIDString:(NSString *)uuidString
{
	//NSArray			*uuidArray	= [NSArray arrayWithObjects:[CBUUID UUIDWithString:uuidString], nil];
	//NSDictionary	*options	= [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:NO] forKey:CBCentralManagerScanOptionAllowDuplicatesKey];
    if (centralManager.state == CBCentralManagerStatePoweredOn)
    {
        [centralManager scanForPeripheralsWithServices:nil options:nil];
        if(LOG)NSLog(@"Start Scan");
    }else{
        if(LOG)NSLog(@"Faile to open the bluetooth\n");
    }
}


- (void) stopScanning
{
    if(LOG)NSLog(@"Stop Scan");
	[centralManager stopScan];
}


- (void)centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI
{
    if(LOG)NSLog(@"Found a BLE Device : %@",peripheral);
    if(LOG)NSLog(@"advertisementData: %@",advertisementData);
    if(LOG)NSLog(@"RSSI: %@",RSSI);
    [BLEDelegate didFoundPeripheral:peripheral advertisementData:advertisementData RSSI:RSSI];
}


#pragma mark -
#pragma mark Connection/Disconnection
/****************************************************************************/
/*						Connection/Disconnection                            */
/****************************************************************************/
- (void) connectPeripheral:(CBPeripheral*)peripheral
{
	if (![peripheral isConnected]) {
		[centralManager connectPeripheral:peripheral options:nil];
        if(LOG)NSLog(@"connectPeripheral: %@",peripheral);
	}
}


- (void) disconnectPeripheral:(CBPeripheral*)peripheral
{
	[centralManager cancelPeripheralConnection:peripheral];
    if(LOG)NSLog(@"disconnectPeripheral: %@",peripheral);
}


- (void) centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral
{
    if(connectedPeripheral != nil) {
        connectedPeripheral = nil;
        connectedService = nil;
        msgCharacteristic = nil;
    }
    connectedPeripheral = peripheral;
    [connectedPeripheral setDelegate:self];
    [connectedPeripheral discoverServices:nil];
    [BLEDelegate didConnectPeripheral:connectedPeripheral];
    if(LOG)NSLog(@"didConnectPeripheral: %@",peripheral);
}

- (void)peripheral:(CBPeripheral *)peripheral didDiscoverServices:(NSError *)error
{
    NSArray *services = [peripheral services];
    NSInteger count = [services count];
    NSInteger i = 0;
    char data[] = QIQU_SERVICE_SHORT_UUID;
    CBUUID *targetUUID = [CBUUID UUIDWithData:[NSData dataWithBytes:data length:sizeof(data)]];
    while (i < count) {
        CBUUID *uuid = [(CBService*)[services objectAtIndex:i] UUID];
        if(LOG)NSLog(@"didDiscoverServices: found uuid%@",uuid);
        if ([uuid isEqual:targetUUID] == YES) {
            connectedService = [services objectAtIndex:i];
            if(LOG)NSLog(@"didDiscoverServices: found 0x18f0");
        }
        i++;
    }
    
    if(LOG)NSLog(@"didDiscoverServices: %@",services);
    
    if(connectedService != nil){
        [peripheral discoverCharacteristics:nil forService:connectedService];
    }
}
- (void)peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(CBService *)service error:(NSError *)error
{
    NSArray *characters = [service characteristics];
    NSInteger count = [characters count];
    NSInteger i = 0;
    char data[] = QIQU_CHARACTERISTIC_TX_UUID;
    CBUUID *targetUUID = [CBUUID UUIDWithData:[NSData dataWithBytes:data length:sizeof(data)]];
    CBCharacteristic *characteristic;
    
    for (characteristic in characters) {
        if(LOG)NSLog(@"didDiscoverCharacteristicsForService: %@",characteristic);
        [peripheral readValueForCharacteristic:characteristic];
        [peripheral setNotifyValue:YES forCharacteristic:characteristic];
    }
    
    while (i < count) {
        CBUUID *uuid = [(CBService*)[characters objectAtIndex:i] UUID];
        if ([uuid isEqual:targetUUID] == YES) {
            msgCharacteristic = [characters objectAtIndex:i];
            if(LOG)NSLog(@"didDiscoverCharacteristicsForService: uuid is %@",uuid);
        }
        i++;
    }
    
}

- (void) centralManager:(CBCentralManager *)central didFailToConnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error
{
    if(LOG)NSLog(@"Attempted connection to peripheral %@ failed: %@", [peripheral name], [error localizedDescription]);
    [BLEDelegate didFailToConnectPeripheral:peripheral error:error];
}


- (void) centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error
{
    if(connectedPeripheral != nil) {
        connectedPeripheral = nil;
    }
    if(LOG)NSLog(@"Disconnect with peripheral: %@",peripheral);

    [BLEDelegate didDisconnectPeripheral:peripheral];

}

- (void) clearDevices
{
    //need disconn ?
    if(connectedPeripheral != nil) {
        connectedPeripheral = nil;
    }
}


- (void) centralManagerDidUpdateState:(CBCentralManager *)central
{
    
    static CBCentralManagerState previousState = -1;
	switch ([centralManager state]) {
		case CBCentralManagerStatePoweredOff:
		{
            if(LOG)NSLog(@"CBCentralManagerStatePoweredOff");
            [self clearDevices];
            
			/* Tell user to power ON BT for functionality, but not on first run - the Framework will alert in that instance. */
            if (previousState != -1) {
                //[discoveryDelegate discoveryStatePoweredOff];
            }
			break;
		}
            
		case CBCentralManagerStateUnauthorized:
		{
			/* Tell user the app is not allowed. */
            if(LOG)NSLog(@"CBCentralManagerStateUnauthorized");
			break;
		}
            
		case CBCentralManagerStateUnknown:
		{
			/* Bad news, let's wait for another event. */
            if(LOG)NSLog(@"CBCentralManagerStateUnknown");
			break;
		}
            
		case CBCentralManagerStatePoweredOn:
		{
            if(LOG)NSLog(@"CBCentralManagerStatePoweredOn");
            [centralManager retrieveConnectedPeripherals];            
            [centralManager scanForPeripheralsWithServices:nil options:nil];
			break;
		}
            
		case CBCentralManagerStateResetting:
		{
            if(LOG)NSLog(@"CBCentralManagerStateResetting");
			[self clearDevices];
			break;
		}
            
        case CBCentralManagerStateUnsupported:
        {
            break;
        }
	}
    
    previousState = [centralManager state];
}

#pragma mark -
#pragma mark Rcv/Snd Data
/****************************************************************************/
/*                               Rcv/Snd                                    */
/****************************************************************************/
- (void) peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error
{
    //if(LOG)NSLog(@"didUpdateValueForCharacteristic:%@:%@", [characteristic UUID],[characteristic value]);
    if(LOG)NSLog(@"RCV data:%@",[characteristic value]);
    NSData *rcvData = [characteristic value];
    if (rcvData != nil) {
        [BLEDelegate didRecieveData:rcvData];
    }
}

- (void)peripheral:(CBPeripheral *)peripheral didUpdateNotificationStateForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error
{
    if(LOG)NSLog(@"didUpdateNotificationStateForCharacteristic:%@:%@", characteristic,error);
}

- (BOOL)sendData:(NSData*)data
{
    if (data == nil) {
        if(LOG)NSLog(@"Sending Failed! Reason: Data was not available.");
        return NO;
    }
    if(connectedPeripheral == nil){
        if(LOG)NSLog(@"Sending Failed! Reason: Peripheral was not available.");
        return NO;
    }
    if (msgCharacteristic == nil) {
        if(LOG)NSLog(@"Sending Failed! Reason: Characteristic was not available.");
        return NO;
    }
    NSInteger len = [data length];
    NSRange range;
    range.length = 0;
    range.location = 0;
    //int i =  0;
    while (range.location < len) {
        if (len - range.location > 20) {
            range.length = 20;
        }else{
            range.length = len - range.location;
        }
        NSData *sendData = [data subdataWithRange:range];
        [connectedPeripheral writeValue:sendData forCharacteristic:msgCharacteristic  type:CBCharacteristicWriteWithoutResponse];
        self.txDelayCounter ++;
        if (self.txDelayCounter >= 3) {
            usleep(20000);
            self.txDelayCounter = 0;
        }
        range.location += range.length;
        //if(LOG)NSLog(@"sendLength:%lu sendData:%@",(unsigned long)range.length,sendData);
    }
    if(LOG)NSLog(@"SendData:%@", data);
    //[connectedPeripheral writeValue:data forCharacteristic:msgCharacteristic  type:CBCharacteristicWriteWithoutResponse];
    return YES;
    
}

- (void)peripheral:(CBPeripheral *)peripheral didWriteValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error
{
    /* When a write occurs, need to set off a re-read of the local CBCharacteristic to update its value */
    //[peripheral readValueForCharacteristic:characteristic];
    if(LOG)NSLog(@"didWriteValueForCharacteristic:%@", [characteristic value]);
    [BLEDelegate didSendData:[characteristic value]];
}




#pragma mark -
#pragma mark Utility
/****************************************************************************/
/*                               Utility                                    */
/****************************************************************************/

- (BOOL)isConnected
{
    if (connectedPeripheral == nil) {
        return NO;
    }else{
        return YES;
    }
}

- (BOOL)isConnectedWithPeripheral:(CBPeripheral*)peripheral
{
    if ([self isConnected] == NO) {
        return NO;
    }
    if ([peripheral isEqual:connectedPeripheral] == YES) {
        return YES;
    }else{
        return NO;
    }
}

- (void)enterBackground
{
    if ([[UIDevice currentDevice] isMultitaskingSupported]) {
        [self setBackgroundRecordingID:[[UIApplication sharedApplication] beginBackgroundTaskWithExpirationHandler:^{}]];
    }
}

- (void)enterForeground
{
    if ([[UIDevice currentDevice] isMultitaskingSupported]) {
        [[UIApplication sharedApplication] endBackgroundTask:[self backgroundRecordingID]];
    }
    
}

- (void)pushNotificationWithMessage:(NSString *)messageString buttonTitle:(NSString *)buttonTitle inSeconds:(NSInteger)delaySeconds
{
    UILocalNotification *notification=[[UILocalNotification alloc] init];
    if (notification!=nil) {
        NSDate *now = [NSDate new];
        notification.fireDate = [now dateByAddingTimeInterval:delaySeconds];
        notification.repeatInterval = 0;
        notification.timeZone = [NSTimeZone defaultTimeZone];
        notification.applicationIconBadgeNumber = 1;//red
        notification.soundName = UILocalNotificationDefaultSoundName;
        notification.alertBody = messageString;
        notification.alertAction = buttonTitle;
        [[UIApplication sharedApplication] scheduleLocalNotification:notification];
    }
    
}

- (void)cleanApplicationIconBadgeNumber
{
    
    [UIApplication sharedApplication].applicationIconBadgeNumber=0;
    
}
@end
